// StressCPU Utility
//
// This utility will consume an arbitrary amount of processor cycles
// that is controlled using a simple slider bar UI.  Multiple processors
// are automatically detected and handled.  The amount of cycles to consume
// specified using the slider is the amount to consume above and beyond what's
// already being consumed.  So if the system is currently at 15% utilization,
// setting this applications slider to 20% increases the system's utilization
// by 20%, for a total of 35%.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using System.Runtime.InteropServices;
using System.Security;

class Win32
{
    [StructLayout(LayoutKind.Explicit)]
    internal struct SYSTEM_INFO
    {
        // union {
        [FieldOffset(0)]  internal uint     dwOemId;
        //   struct {
        [FieldOffset(0)]  internal ushort   wProcessorArchitecture;
        [FieldOffset(2)]  internal ushort   wReserved;
        //   }
        // }
        [FieldOffset(4)]  internal uint     dwPageSize;
        [FieldOffset(8)]  internal uint     lpMinimumApplicationAddress;
        [FieldOffset(12)] internal uint     lpMaximumApplicationAddress;
        [FieldOffset(16)] internal uint     dwActiveProcessorMask;
        [FieldOffset(20)] internal uint     dwNumberOfProcessors;
        [FieldOffset(24)] internal uint     dwProcessorType;
        [FieldOffset(28)] internal uint     dwAllocationGranularity;
        [FieldOffset(32)] internal ushort   wProcessorLevel;
        [FieldOffset(34)] internal ushort   wProcessorRevision;
    }

    [DllImport("kernel32.dll"), SuppressUnmanagedCodeSecurity]
    internal static extern void GetSystemInfo( out SYSTEM_INFO si );

    [DllImport("kernel32.dll"), SuppressUnmanagedCodeSecurity]
    internal static extern bool SetPriorityClass( uint hProcess, uint dwPriorityClass );

    [DllImport("kernel32.dll"), SuppressUnmanagedCodeSecurity]
    internal static extern uint GetCurrentProcess();

    internal enum PriorityClass
    {
        Realtime,
        High,
        AboveNormal,
        Normal,
        BelowNormal,
        Idle
    }

    private static uint[] PriorityClassValues =
    {
        // The following values are taken from winbase.h,
        // and coincide in index order to the PriorityClass
        // enum above.  This is done this way because assigning
        // the values directly to the enum members above
        // causes Enum.GetNames to return them in ascending numeric
        // order, not the order they were declared above.  Because
        // I want them displayed in the dialog box in highest-to-lowest
        // priority order, I need to use unqualified enums above
        // and then map them onto the correct Win32 constants myself.
        //
        0x00000100,
        0x00000080,
        0x00008000,
        0x00000020,
        0x00004000,
        0x00000040
    };

    internal static void SetProcessPriorityClass( PriorityClass pc )
    {
        SetPriorityClass(GetCurrentProcess(), PriorityClassValues[(int)pc]);
    }
}


namespace stresscpu
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
        private volatile bool       currentlyRunning = false;
        private volatile bool       exitPending = false;
        private volatile int        sliderPos = 0;
        private ManualResetEvent    wakeEvent = new ManualResetEvent(false);
        private Thread[]            mipsThreads;
        private int                 processorCount;

        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.TrackBar sliderBar;
        private System.Windows.Forms.Label usageLabel;
        private System.Windows.Forms.Button startStopButton;
        private System.Windows.Forms.Button exitButton;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.ComboBox processPriority;
        private System.Windows.Forms.ComboBox threadPriority;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

            // Populate the process and thread priority combo boxes.
            //
            foreach( string pc in Enum.GetNames(typeof(Win32.PriorityClass)) )
            {
                processPriority.Items.Add(pc);
            }

            string[] priorityNames = Enum.GetNames(typeof(ThreadPriority));
            Array.Reverse(priorityNames);

            foreach( string tp in priorityNames )
            {
                threadPriority.Items.Add(tp);
            }

            // Select the default Normal priorities.
            //
            processPriority.SelectedIndex = processPriority.FindString(Win32.PriorityClass.Normal.ToString());
            threadPriority.SelectedIndex = threadPriority.FindString(ThreadPriority.Normal.ToString());

            // Determine how many processors are installed.
            //
			Win32.SYSTEM_INFO si;
            Win32.GetSystemInfo(out si);
            processorCount = (int)si.dwNumberOfProcessors;

            // Start one thread per processor.
            //
            mipsThreads = new Thread[processorCount];
            ThreadStart threadProc = new ThreadStart(MipsThreadProc);

            for( int n = 0; n < processorCount; n++ )
            {
                mipsThreads[n] = new Thread(threadProc);
                mipsThreads[n].Priority = (ThreadPriority)Enum.Parse(typeof(ThreadPriority), (string)threadPriority.SelectedItem);
                mipsThreads[n].Name = string.Format("MIPS-{0}", n);
                mipsThreads[n].IsBackground = true; // Not important enough to hold process running.
                mipsThreads[n].Start();
            }

            // Update the caption to include the processor count.
            //
            string caption = this.Text;
            this.Text =
                string.Format( "{0} ({1} {2})", caption, processorCount,
                               (processorCount > 1 ? "processors" : "processor") );
		}

        private void MipsThreadProc()
        {
            try
            {
                // Until we're told the program is shutting down...
                //
                while( !exitPending )
                {
                    // Wait for a command from the UI.
                    //
                    if( wakeEvent.WaitOne() )
                    {
                        // If we've just woken up, check to see why.
                        // If an exit isn't pending, then go into the spin
                        // loop until we're told to stop spinning and go back
                        // idle, or exit.
                        //
                        while( !exitPending && currentlyRunning )
                        {
                            // Calculate the portion of a 1 second period that
                            // we want to occupy the CPU for.  Then do a busy
                            // wait for that period of time, followed by a
                            // blocking sleep for the remainder.
                            //
                            double percentBusy = (double)sliderPos / 100.0;
                            double percentIdle = (double)1.0 - percentBusy;
                            int  idlePeriod = (int)(percentIdle * 1000);
                            DateTime stopTime = DateTime.Now.AddMilliseconds(1000 * percentBusy);

                            while( DateTime.Now < stopTime )
                            {
                            }

                            Thread.Sleep(idlePeriod);
                        }
                    }
                }
            }
            catch
            {
            }
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.label1 = new System.Windows.Forms.Label();
            this.usageLabel = new System.Windows.Forms.Label();
            this.sliderBar = new System.Windows.Forms.TrackBar();
            this.startStopButton = new System.Windows.Forms.Button();
            this.exitButton = new System.Windows.Forms.Button();
            this.label2 = new System.Windows.Forms.Label();
            this.label3 = new System.Windows.Forms.Label();
            this.processPriority = new System.Windows.Forms.ComboBox();
            this.threadPriority = new System.Windows.Forms.ComboBox();
            ((System.ComponentModel.ISupportInitialize)(this.sliderBar)).BeginInit();
            this.SuspendLayout();
            // 
            // label1
            // 
            this.label1.Location = new System.Drawing.Point(8, 72);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(160, 23);
            this.label1.TabIndex = 4;
            this.label1.Text = "Processor cycles to consume:";
            // 
            // usageLabel
            // 
            this.usageLabel.Location = new System.Drawing.Point(168, 72);
            this.usageLabel.Name = "usageLabel";
            this.usageLabel.Size = new System.Drawing.Size(40, 23);
            this.usageLabel.TabIndex = 5;
            this.usageLabel.Text = "0%";
            // 
            // sliderBar
            // 
            this.sliderBar.Location = new System.Drawing.Point(8, 96);
            this.sliderBar.Maximum = 95;
            this.sliderBar.Name = "sliderBar";
            this.sliderBar.Size = new System.Drawing.Size(264, 45);
            this.sliderBar.TabIndex = 6;
            this.sliderBar.TickFrequency = 24;
            this.sliderBar.Scroll += new System.EventHandler(this.sliderBar_Scroll);
            // 
            // startStopButton
            // 
            this.startStopButton.Location = new System.Drawing.Point(47, 144);
            this.startStopButton.Name = "startStopButton";
            this.startStopButton.TabIndex = 7;
            this.startStopButton.Text = "Start";
            this.startStopButton.Click += new System.EventHandler(this.startStopButton_Click);
            // 
            // exitButton
            // 
            this.exitButton.Location = new System.Drawing.Point(159, 144);
            this.exitButton.Name = "exitButton";
            this.exitButton.TabIndex = 8;
            this.exitButton.Text = "Exit";
            this.exitButton.Click += new System.EventHandler(this.exitButton_Click);
            // 
            // label2
            // 
            this.label2.Location = new System.Drawing.Point(8, 8);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(120, 23);
            this.label2.TabIndex = 0;
            this.label2.Text = "Process priority class:";
            // 
            // label3
            // 
            this.label3.Location = new System.Drawing.Point(144, 8);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(120, 23);
            this.label3.TabIndex = 2;
            this.label3.Text = "Thread priority:";
            // 
            // processPriority
            // 
            this.processPriority.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.processPriority.Location = new System.Drawing.Point(16, 32);
            this.processPriority.Name = "processPriority";
            this.processPriority.Size = new System.Drawing.Size(121, 21);
            this.processPriority.TabIndex = 1;
            this.processPriority.SelectedIndexChanged += new System.EventHandler(this.processPriority_SelectedIndexChanged);
            // 
            // threadPriority
            // 
            this.threadPriority.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.threadPriority.Location = new System.Drawing.Point(152, 32);
            this.threadPriority.Name = "threadPriority";
            this.threadPriority.Size = new System.Drawing.Size(121, 21);
            this.threadPriority.TabIndex = 3;
            this.threadPriority.SelectedIndexChanged += new System.EventHandler(this.threadPriority_SelectedIndexChanged);
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(280, 174);
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                          this.threadPriority,
                                                                          this.processPriority,
                                                                          this.label3,
                                                                          this.label2,
                                                                          this.exitButton,
                                                                          this.startStopButton,
                                                                          this.sliderBar,
                                                                          this.usageLabel,
                                                                          this.label1});
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "Form1";
            this.Text = "CPU Stress Utility";
            this.Closing += new System.ComponentModel.CancelEventHandler(this.Form1_Closing);
            ((System.ComponentModel.ISupportInitialize)(this.sliderBar)).EndInit();
            this.ResumeLayout(false);

        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

        private void sliderBar_Scroll(object sender, System.EventArgs e)
        {
            // sliderPos is the cached value of the slider position, and is
            // updated only within this event handler.  This allows the MIPS thread
            // procedure to read the slider position dynamically without having to interact
            // with the control; which internally uses SendMessage to query the slider
            // position (which would serialize the MIPS threads on the UI thread's
            // message queue and cause a gazillion context switches per second).
            //
            sliderPos = sliderBar.Value;
            usageLabel.Text = string.Format("{0}%", sliderPos);
        }

        private void startStopButton_Click(object sender, System.EventArgs e)
        {
            if( currentlyRunning )
            {
                // Reset the event before stopping the mips sucking
                // threads, or there's the risk that they'll stop
                // sucking mips and reach the top of their while loop
                // before we can get this event reset.
                //
                wakeEvent.Reset();
                currentlyRunning = false;
                startStopButton.Text = "Start";
            }
            else
            {
                // Set things up before unleashing the mips sucking
                // thread(s).
                //
                currentlyRunning = true;
                startStopButton.Text = "Stop";
                wakeEvent.Set();
            }
        }

        // This isn't strictly necessary, but I like to take my threads
        // through an orderly shutdown before exiting the application.
        //
        private void exitButton_Click(object sender, System.EventArgs e)
        {
            Shutdown();
            Application.Exit();
        }

        private void Form1_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Shutdown();
        }

        private void Shutdown()
        {
            if( currentlyRunning )
            {
                // The mips sucker thread(s) are running right now,
                // so reset the wake event, then tell them to stop,
                // then give them a chance to get settled.
                //
                wakeEvent.Reset();
                currentlyRunning = false;
                Thread.Sleep(250);
            }
            else
            {
                // The mips sucker thread(s) are idle right now,
                // so we'll just need to wake them up.
            }

            exitPending = true;
            wakeEvent.Set();

            foreach( Thread t in mipsThreads )
            {
                t.Join();
            }
        }

        private void processPriority_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            // Adjust the process priority class.
            //
            Win32.PriorityClass pc = (Win32.PriorityClass)
                Enum.Parse( typeof(Win32.PriorityClass),
                            (string)processPriority.SelectedItem );

            Win32.SetProcessPriorityClass(pc);
        }

        private void threadPriority_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            // Adjust each thread's priority.  The mipsThread array is only null
            // when this event occurs as a result of the form's initialization of
            // the selected item.
            //
            if( mipsThreads == null )
                return;

            ThreadPriority tp = (ThreadPriority)
                Enum.Parse( typeof(ThreadPriority),
                            (string)threadPriority.SelectedItem );

            foreach( Thread t in mipsThreads )
            {
                t.Priority = tp;
            }
        }
	}
}
